﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using System.Threading.Tasks;
using System.IO;

namespace CosmoZ
{
	public partial class CosmoZWrapper
	{
		#region CONSTRUCTOR_DESTRUCTOR
		public CosmoZWrapper()
		{
		}
		~CosmoZWrapper()
		{
		}
		#endregion

		public MCAInfo get_mcainfo()
		{
			MCAInfo mcaInfo;
			csz_get_mcaheader(out mcaInfo);

			int recvlen = (int)(mcaInfo.maxch * mcaInfo.resolution);
			mcaInfo.data = new UInt32[recvlen];

			unsafe
			{
				fixed (UInt32* p = mcaInfo.data)
				{
					IntPtr ptr = (IntPtr)p;
					csz_get_mcadata(ptr, recvlen * sizeof(UInt32));
				}
			}

			return mcaInfo;
		}


		public void mca_start(UInt32 goaltime, bool elapsed_mode, bool use_upp)
		{
			csz_mca_start(goaltime, elapsed_mode, use_upp);
		}

		public void mca_stop()
		{
			csz_mca_stop();
		}

		public void mca_clear()
		{
			csz_mca_clear();
		}

		public CosmoZStatus getStatus()
		{
			CosmoZStatus result = new CosmoZStatus();

			result.maxch = adc_maxch();
			result.analogChInfo = new ANALOG_CH_INFO[result.maxch];
			result.channelTrigger = new ChannelTriger[result.maxch];
			for (int i = 0; i < result.maxch; i++)
			{
				result.analogChInfo[i] = new ANALOG_CH_INFO();
				result.channelTrigger[i] = new ChannelTriger();
				result.analogChInfo[i].input_min_voltage = -0.5;
				result.analogChInfo[i].input_max_voltage = 0.5;
				result.analogChInfo[i].filter_fc = 50;
				result.analogChInfo[i].gain = 1;
				result.channelTrigger[i].trigType = csz_trig_gettype(i + 1);
				result.channelTrigger[i].trigLevel = csz_trig_getadcval(i + 1);
				result.channelTrigger[i].isGlobalCapture = csz_trig_get_gcap(i + 1);
				result.channelTrigger[i].cps = csz_trig_rate(i + 1);
				result.channelTrigger[i].pedestal = csz_trig_pedestal(i + 1);
				result.channelTrigger[i].plsh = csz_trig_pulseheight(i + 1);
			}

			result.OrTrig = trig_is_ortrig();
			result.pretrig = pretrig_get();
			result.posttrig = posttrig_get();
			result.fixedPosttrig = posttrig_is_fixed();
			result.FPGATime = csz_time48();
			result.GPSTime = 0;
			result.Temperature = csz_xadc_tempe();
			result.VccAux = csz_xadc_vccaux();
			result.VccDdro = csz_xadc_vccddro();
			result.VccInt = csz_xadc_vccint();

			return result;
		}

		public CosmoZStatus2 remote_status()
		{
			return remote_get_status();
		}

		[StructLayout(LayoutKind.Sequential)]

		/// 通信上のプロトコルとしての構造体
		public unsafe struct UPPStatus
		{
			public int length; /// この構造体のサイズ
			public int maxch;
			public int freq;
			public fixed UInt32 ctrl[32]; // 50
			public fixed UInt32 tfa_tau[32]; // 51
			public fixed UInt32 tfa_pz[32]; // 51
			public fixed UInt32 tfa_a[32]; // 52
			public fixed UInt32 tfa_b[32]; // 52
			public fixed UInt32 cfd_ratio[32]; // 53
			public fixed UInt32 cfd_tau[32]; // 53
			public fixed UInt32 trapezoid_m1[32]; // 55
			public fixed UInt32 trapezoid_m2[32]; // 56
			public fixed UInt32 trapezoid_l[32]; // 57
			public fixed UInt32 trapezoid_k[32]; // 57
			public fixed UInt32 trapezoid_ctrl[32]; // 58
			public fixed UInt32 trig_delay[32]; // 59
			public fixed UInt32 trig_width[32]; // 59
			public fixed UInt32 upper_discri[32]; // 60
			public fixed UInt32 lower_discri[32]; // 60
			public fixed UInt32 coinc_matrix[32];
			public fixed UInt32 veto_matrix[32];
		}

		/// <summary>
		/// ハードウェアと通信して、C#で安全なUPPStatus2構造体に整形して返す
		/// </summary>
		/// <returns></returns>
		public UPPStatus2 get_upp_status()
		{
			UPPStatus commStat = csz_get_upp_status();
			UPPStatus2 stat;
			CosmoZStatus2 stat2 = remote_get_status();
			stat.maxch = commStat.maxch;
			stat.freq = commStat.freq;
			stat.ch = new UPPChannelStatus[stat.maxch];
			for (int c = 0; c < stat.maxch; c++)
			{
				stat.ch[c] = new UPPChannelStatus();
				unsafe
				{
					stat.ch[c].ctrl = (UInt16)commStat.ctrl[c]; // 50
					stat.ch[c].tfa_tau = (UInt16)commStat.tfa_tau[c]; // 51
					stat.ch[c].tfa_pz = (UInt16)commStat.tfa_pz[c]; // 51
					stat.ch[c].tfa_a = (UInt16)commStat.tfa_a[c]; // 52
					stat.ch[c].tfa_b = (UInt16)commStat.tfa_b[c]; // 52
					stat.ch[c].cfd_ratio = (UInt16)commStat.cfd_ratio[c]; // 53
					stat.ch[c].cfd_tau = (UInt16)commStat.cfd_tau[c]; // 53
					stat.ch[c].trapezoid_m1 = commStat.trapezoid_m1[c]; // 55
					stat.ch[c].trapezoid_m2 = commStat.trapezoid_m2[c]; // 56
					stat.ch[c].trapezoid_l = (UInt16)commStat.trapezoid_l[c]; // 57
					stat.ch[c].trapezoid_k = (UInt16)commStat.trapezoid_k[c]; // 57
					stat.ch[c].trapezoid_ctrl = commStat.trapezoid_ctrl[c]; // 58
					stat.ch[c].trig_delay = (UInt16)commStat.trig_delay[c]; // 59
					stat.ch[c].trig_width = (UInt16)commStat.trig_width[c]; // 59
					stat.ch[c].upper_discri = (UInt16)commStat.upper_discri[c]; // 60
					stat.ch[c].lower_discri = (UInt16)commStat.lower_discri[c]; // 60
					stat.ch[c].coinc_matrix = commStat.coinc_matrix[c];
					stat.ch[c].veto_matrix = commStat.veto_matrix[c];
					stat.ch[c].input_min_voltage = stat2.input_min_voltage[c];
					stat.ch[c].input_max_voltage = stat2.input_max_voltage[c];
					stat.ch[c].fullscale = stat2.fullscale[c];
				}
			}
			return stat;
		}


		////////////////////////////////////////////////////////////////

		public enum EVENTCAP_MODE { EVENTCAP_MODE_TIME = 0, EVENTCAP_MODE_COUNT = 1 };

		private const int REGOFS_CAPTLEN = 16;  //!< FPGAの内蔵レジスタ16
		private const int REGOFS_CAPTCH = 17;  //!< FPGAの内蔵レジスタ17
		private const int REGOFS_CAPTADDR = 18;  //!< End Address。どこまでデータが書き込まれているか
		private const int REGOFS_CAPTTIME = 19;  //!< FPGAの内蔵レジスタ19
		private const int REGOFS_CAPTCTRL = 20;  //!< FPGAの内蔵レジスタ20
		private const int REGOFS_CAPTCOUNT = 21;  //!< FPGAの内蔵レジスタ21
		private const int REGOFS_CAPTDLY = 22;  //!< FPGAの内蔵レジスタ22
		private const int REGOFS_CAPTITER = 23;  //!< FPGAの内蔵レジスタ23
		private const int REGOFS_CAPTBADDR = 24;  //!< Bottom Address オーバーラップしたときに使う

		private const UInt32 FLAG_CAPTURE_START = (1 << 0); // キャプチャを実行するユーザコマンドが与えられた
		private const UInt32 FLAG_CAPTURE_AUTO = (1 << 1);
		private const UInt32 FLAG_CAPTURE_RUN = (1 << 2); // キャプチャを実行中(トリガ待ちでまだ開始されていない場合は0)
		private const UInt32 FLAG_CAPTURE_ERROR = (1 << 3);
		private const UInt32 FLAG_CAPTURE_OSCILLO = (1 << 8);
		private const UInt32 FLAG_CAPTURE_EVENT = (2 << 8);
		private const UInt32 FLAG_CAPTURE_MESUNIT = (3 << 8);
		private const UInt32 FLAG_CAPTURE_LONGCAP = (4 << 8);

		public EVCAP_STATUS get_inprogress_capture()
		{
			EVCAP_STATUS evstat;
			csz_evcap_status(out evstat);
			return evstat;
		}

		//public RawWaves.RawWaves GetEvcapData(EVCAP_STATUS evstat)
		//{
		//	byte[] data = new byte[evstat.eaddr - 0x20000480];
		//	self.mem_read(0x480, data.Length, data);
		//	int p = 0;
		//	int evcount = 0;

		//	int[] lengths = new int[evstat.maxch];
		//	for(int i=0; i<evstat.maxch; i++)
		//	{
		//		lengths[i] = 0;
		//	}
		//	Int64 StartTime = 0;
		//	Int64 EndTime = 0;
		//	while (p < data.Length)
		//	{
		//		int datalen = data[p + 2] + (data[p + 3] << 8);
		//		datalen = (datalen + 3) & ~3;
		//		ushort key = (ushort)(data[p] + (data[p + 1] << 8));
		//		if ((key & 0xfe0f) != 0x0001) // キーワードが01でなければ、壊れている
		//		{
		//			break;
		//		}
		//		int ch = (key & 0x1f0) >> 4; // 記録されたADCチャネル(1～32)
		//		lengths[ch] += datalen;

		//		p += (datalen + 12) * 2;
		//		evcount++;
		//	}
		//	RawWaves.RawWaves rw = new RawWaves.RawWaves(evstat.maxch, 0);
		//	int maxlength = 0;
		//	for (int ch = 0; ch < evstat.maxch; ch++)
		//	{
		//		if (lengths[ch] > maxlength) maxlength = lengths[ch];
		//	}
		//	rw.length = maxlength;
		//	for (int ch = 0; ch < evstat.maxch; ch++)
		//	{
		//		rw.ch[ch] = new RawWave();
		//		if (lengths[ch] == 0)
		//		{
		//			rw.ch[ch].data = null;
		//			rw.ch[ch].time = null;
		//			rw.ch[ch].fullscale = 65535;
		//			rw.ch[ch].minvolt = -0.5;
		//			rw.ch[ch].maxvolt = 0.5;
		//			rw.ch[ch].cstart = null;
		//			rw.ch[ch].clen = null;
		//			rw.ch[ch].plsh = null;
		//			rw.ch[ch].length = 0;
		//			rw.ch[ch].samplingType = RawWave.SamplingType.NoData;
		//		}
		//		else
		//		{
		//			rw.ch[ch].data = new ushort[lengths[ch]];
		//			rw.ch[ch].time = new Int64[lengths[ch]];
		//			rw.ch[ch].fullscale = 65535;
		//			rw.ch[ch].minvolt = -0.5;
		//			rw.ch[ch].maxvolt = 0.5;
		//			rw.ch[ch].cstart = new int[lengths[ch]];
		//			rw.ch[ch].clen = new int[lengths[ch]];
		//			rw.ch[ch].plsh = new int[lengths[ch]];
		//			rw.ch[ch].length = lengths[ch];
		//			rw.ch[ch].samplingType = RawWave.SamplingType.Event;
		//		}
		//	}

		//	evcount = 0;
		//	int[] counts = new int[evstat.maxch];
		//	p = 0;
		//	for (int i = 0; i < evstat.maxch; i++)
		//	{
		//		lengths[i] = 0;
		//		counts[i] = 0;
		//	}
		//	while (p < data.Length)
		//	{
		//		int datalen = data[p + 2] + ((int)data[p + 3] << 8);
		//		datalen = (datalen + 3) & ~3;
		//		ushort key = (ushort)(data[p] + (data[p + 1] << 8));
		//		Int64 time = (Int64)(data[p + 4] + ((Int64)data[p + 5] << 8) + ((Int64)data[p + 6] << 16) + ((Int64)data[p + 7] << 24) + ((Int64)data[p + 8] << 32) + ((Int64)data[p + 9] << 40));
		//		if(evcount == 0)
		//		{
		//			StartTime = time;
		//		}
		//		if ((key & 0xfe0f) != 0x0001) // キーワードが01でなければ、壊れている
		//		{
		//			break;
		//		}
		//		int ch = (key & 0x1f0) >> 4; // 記録されたADCチャネル(1～32)

		//		rw.ch[ch].cstart[counts[ch]] = lengths[ch];
		//		rw.ch[ch].clen[counts[ch]] = datalen;
		//		rw.ch[ch].plsh[counts[ch]] = (data[p + 10] + ((int)data[p + 11] << 8)) / 16;
		//		for (int i=0;i< datalen; i++)
		//		{
		//			rw.ch[ch].data[lengths[ch] + i] = (UInt16)(data[p + 24 + i * 2] + (data[p + 24 + i * 2 + 1] << 8));
		//			rw.ch[ch].time[lengths[ch] + i] = (time - StartTime) * 10000 + 1000000 * (Int64)i / evstat.SamplingFreqMHz;
		//		}
		//		lengths[ch] += datalen;
		//		EndTime = (time - StartTime) * 10000 + 1000000 * (Int64)datalen / evstat.SamplingFreqMHz;

		//		p += (datalen + 12) * 2;
		//		evcount++;
		//		counts[ch]++;
		//	}

		//	rw.additionalInfo.waveStat = AdditionalInfo.WaveStatus.EVENTRUN;
		//	rw.additionalInfo.samplingFreqMHz = evstat.SamplingFreqMHz;
		//	rw.additionalInfo.capturedDateTime = CosmoZ.FromUnixTime32(evstat.UnixStartTime);
		//	rw.SetTimeRange(0, EndTime);

		//	return rw;
		//}

		private static int GetSamplingRateFromAdccmd(UInt32 adccmd)
		{
			int freq, div;

			switch ((adccmd >> 8) & 3)
			{
				case 0:
					freq = 80;
					break;
				case 1:
					freq = 100;
					break;
				case 2:
					freq = 125;
					break;
				default:
					freq = 100;
					break;
			}
			div = (int)((adccmd >> 16) & 255) + 1;
			return freq / div;
		}
	}
}
