﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CosmoZ
{
	//! @brief Cosmo-Zのインタフェースに使うポート
	public enum INTERFACE_PORT
	{
		INTERFACE_NONE = 0, //!< 接続していない
		INTERFACE_ZYNQ = 1, //!< ZYNQのダイレクト接続
		INTERFACE_TCPIP = 2, //!< TCP/IP経由での接続
		INTERFACE_PCIE = 3, //!< PCI Expressでの接続(未対応)
		INTERFACE_USBUART = 4, //!< USB UARTでの接続(未対応)
		INTERFACE_USB3TKD = 5, //!< USB 3.0 TKDNプロトコルでの接続(未対応)
	};

	public enum CAPTURE_STATUS
	{
		CAPSTAT_IDLE,            //!< 待機状態
		CAPSTAT_TRIGWAIT,        //!< トリガ待ち状態(通常サンプリング)
		CAPSTAT_RUN,             //!< 実行中(通常サンプリング)
		CAPSTAT_OSCILLO,         //!< オシロモード
		CAPSTAT_MESUNIT,         //!< 計測ユニット
		CAPSTAT_EVENT,           //!< イベント実行中
		CAPSTAT_MESDONE,         //!< 計測ユニット終了
		CAPSTAT_EVDONE,          //!< イベント計測終了
		CAPSTAT_LONGCAP          //!< 長時間キャプチャ
	};

	public enum CAPTURE_RESULT
	{
		CAPRESULT_SUCCESS,         //!< キャプチャは成功した
		CAPRESULT_ABORTED,         //!< キャプチャはユーザからの指示で中断された
		CAPRESULT_ERROR,           //!< データの転送が間に合わなかったなどの理由で失敗した
		CAPRESULT_ANOTHERRUN,      //!< 他のキャプチャが動作しているため、波形キャプチャを開始できない
		CAPRESULT_TRIGGER_TIMEOUT, //!< トリガ待ち状態のままで、タイムアウトした
		CAPRESULT_CORRUPTED,       //!< 取得したデータが壊れている
	};

	public enum CAPTURE_TRIGGER
	{
		CAPTRIG_AUTO,           //!< オシロのAUTOモードのように、少しだけトリガを待ってからキャプチャ
		CAPTRIG_NORMAL,         //!< トリガが来るまで待ってからキャプチャする。オシロのNORMALモードに相当
		CAPTRIG_IMMEDIATE,      //!< 即座にキャプチャを開始する
		CAPTRIG_LONG,           //!< 長時間キャプチャ
		CAPTRIG_MESUNIT,        //!< イベントモードでのキャプチャ(秒数で指定)
		CAPTRIG_EVENT,          //!< イベントモードでのキャプチャ(イベント数で指定)
		CAPTRIG_LAST            //!< 最後に表示した波形を再取得
	};

	public enum TRIGTYPE
	{
		TRIGTYPE_OFF = 0,           //!< トリガは発生しない
		TRIGTYPE_UPPER = 1,         //!< 指定値以上ならトリガ発生
		TRIGTYPE_LOWER = 2,         //!< 指定値以下ならトリガ発生
		TRIGTYPE_CROSS = 3,         //!< 指定値を横切ったらトリガ発生
		TRIGTYPE_DISCRI = 4,        //!< 通常の値より指定値分下回ったらトリガ発生
		TRIGTYPE_RISE = 5,          //!< 指定値を横切って立ち上がったらトリガ発生
		TRIGTYPE_FALL = 6,          //!< 指定値を横切って立ち下がったらトリガ発生
		TRIGTYPE_LINKAGE = 7,       //!< 他のチャネルがヒットしたらトリガ発生
		TRIGTYPE_USER = 8,          //!< ユーザ定義による
		TRIGTYPE_ERROR = -1          //!< エラー
	};

	public enum EXTTRIGTYPE
	{
		EXTTRIGTYPE_OFF,        //!< トリガは発生しない
		EXTTRIGTYPE_RISE,       //!< 立ち上がりでトリガ発生
		EXTTRIGTYPE_FALL,       //!< 立ち下がりでトリガ発生
		EXTTRIGTYPE_HIGH,       //!< Hレベルでトリガ発生
	};

	public enum MCAStatus { CLEAR = 0, DONE = 1, RUN = 2, STOP = 3 };

}
