﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.InteropServices;

namespace CosmoZ
{
	[StructLayout(LayoutKind.Sequential)]
	public class ADC_INFO
	{
		public Int32 basefreq;           //!< デシメーションされる前のベースとなるクロック周波数
		public Int32 decimation_ratio;   //!< デシメーション比
		public double sampling_freq_mhz;  //!< デシメーションされたサンプリング周波数
		public Int32 adc_num;            //!< ADCの数 (Cosmo-Zならばチャネル数の1/4になる)
		public bool testmode;           //!< テストモードか否か
		public Int32 device_max_freq;    //!< ADCの最高クロック周波数
		public Int32 device_resolution;  //!< ADCの持つ本来の分解能(bit)
		public Int32 current_resolution; //!< 現在の分解能(bit)
		public char[,] device_type; //!< ADCの型番が文字列で入る。最大8個のADCを扱える。
		public Int32[] maxfreq;         //!< 各ADCの最大周波数。最大8個
		public Int32 clkfreq;            //!< ADCのクロック周波数
		public Int32 div;                //!< 分周比。デシメーションともいう。
		public Int32 cif;                //!< CIFフィルタを使っている場合は、CIFのデシメーション比。使用していなければ1
		public ADC_INFO()
		{
			basefreq = 80;
			decimation_ratio = 1;
			sampling_freq_mhz = 80;
			adc_num = 2;
			testmode = false;
			device_max_freq = 80;
			device_resolution = 12;
			current_resolution = 12;
			device_type = new char[8, 32];
			maxfreq = new Int32[8];
			clkfreq = 80;
			div = 1;
			cif = 0;
		}
	};

	[StructLayout(LayoutKind.Sequential)]
	public struct ANALOG_CH_INFO
	{
		public double input_min_voltage;  //!< 最小入力電圧(V)
		public double input_max_voltage;  //!< 最大入力電圧(V)
		public double filter_fc;          //!< アナログフロントエンドのLPFのカットオフ周波数(MHz)
		public double gain;               //!< アナログフロントエンドのゲイン。真数で示す。
	};

	[StructLayout(LayoutKind.Sequential, Pack = 4)]
	public struct CAPTURE_INFO
	{
		public CAPTURE_RESULT result; //!< キャプチャ実行結果
		public UInt32 chmask;       //!< キャプチャしたチャネル
		public Int32 chcount;           //!< キャプチャしたチャネル数
		public Int32 length;            //!< キャプチャ長
		public Int32 iteration;         //!< 繰り返し回数
		public Int32 maxseq;            //!< 最大シーケンス長
		public CAPTURE_TRIGGER trig;  //!< キャプチャトリガの種類
		public Int32 padding1;
		public UInt64 start_time;     //!< 開始時刻
		public UInt64 end_time;       //!< 終了時刻
		public double sampling_rate;  //!< サンプリングレート
		public Int32 adc_freq;          //!< 実行開始時のADC周波数
		public Int32 adc_div;           //!< 実行開始時のデシメーション比
		public Int32 resolution;        //!< 分解能(bit単位)
		public UInt32 gps_time;     //!< 実行開始時のGPS時刻
		public UInt32 gps_10ns;     //!< 実行開始時のGPS時刻 10ns単位
		public UInt32 gps_maxcount; //!< 実行開始時のGPS最大カウント
		public UInt64 start_time48; //!< 実行開始時の48bitタイムスタンプ
		public UInt32 top_addr;     //!< 格納される物理アドレスの先頭
		public UInt32 bot_addr;     //!< 格納される物理アドレスの最後尾
		public UInt32 last_addr;    //!< 格納された物理アドレスのlast
									// 以下はイベントキャプチャで使う
		public Int32 mesp;              //!< 現在読み出し中のデータのポインタ
		public Int32 evcount;           //!< 現在読み出し中のデータの数
		public Int32 prev_capaddr; //!< イベントキャプチャのデータをここまで読んだポインタ
		public UInt32 start_addr;   //!< イベントキャプチャのデータが入っている先頭アドレス
	};

	[StructLayout(LayoutKind.Sequential)]
	public struct MCAInfo
	{
		public UInt32 length;
		public MCAStatus mcaStatus;
		public UInt32 maxch;
		public UInt32 resolution;
		public UInt32 elapsed_mode;
		public UInt32 upp_mode;
		public UInt32 StartTime; // 開始時刻
		public UInt32 EndTime; // 終了時刻
		public UInt32 CurrentTime; // 現在時刻
		public UInt32 ElapsedTime; // 経過時間
		public UInt32 real_time;
		public UInt32 preset_time;
		public UInt32 live_time;
		public UInt32 padding;
		public UInt32 totalcount_l;
		public UInt32 totalcount_h;
		public String Comment;
		public CosmoZStatus2 stat;
		public UInt32[] data;
	}

	public struct ChannelTriger
	{
		public TRIGTYPE trigType;
		public int trigLevel;
		public bool isGlobalCapture;
		public int cps;
		public int pedestal;
		public int plsh;
	};

	public struct CosmoZStatus
	{
		public int maxch;
		public ANALOG_CH_INFO[] analogChInfo;
		public ChannelTriger[] channelTrigger;
		public bool OrTrig;
		public int pretrig;
		public int posttrig;
		public bool fixedPosttrig;
		public UInt64 FPGATime;
		public UInt64 GPSTime;
		public double Temperature;
		public double VccAux;
		public double VccInt;
		public double VccDdro;
	};

	[StructLayout(LayoutKind.Sequential)]
	public unsafe struct CosmoZStatus2
	{
		public int Ok;
		public int maxch;
		public int OrTrig;
		public int pretrig;
		public int posttrig;
		public int fixedPosttrig;
		public int sampling_freq_hz;
		public UInt64 FPGATime;
		public UInt64 GPSTime;
		public double Temperature;
		public double VccAux;
		public double VccInt;
		public double VccDdro;
		public fixed int cps[32];
		public fixed int pedestal[32];
		public fixed int plsh[32];
		public fixed double input_min_voltage[32];  //!< 最小入力電圧(V)
		public fixed double input_max_voltage[32];  //!< 最大入力電圧(V)
		public fixed int fullscale[32];
		public fixed UInt32 trigType[32];
		public fixed int trigLevel[32];
		public fixed int isGlobalCapture[32];
		public fixed int extTrigType[4];
	}

	[StructLayout(LayoutKind.Sequential)]
	public struct EVCAP_STATUS
	{
		public CAPTURE_STATUS capture_stat;
		public bool run;
		public int maxch;
		public UInt32 chmask;
		public UInt32 count;
		public UInt32 time;
		public UInt32 length;
		public UInt32 iter;
		public UInt32 UnixStartTime;
		public Int32 SamplingFreqMHz;
		public UInt32 eaddr;
		public UInt32 baddr;
	}

	public struct UPPChannelStatus
	{
		public UInt16 ctrl; // 50
		public UInt16 tfa_tau; // 51
		public UInt16 tfa_pz; // 51
		public UInt16 tfa_a; // 52
		public UInt16 tfa_b; // 52
		public UInt16 cfd_ratio; // 53
		public UInt16 cfd_tau; // 53
		public UInt32 trapezoid_m1; // 55
		public UInt32 trapezoid_m2; // 56
		public UInt16 trapezoid_l; // 57
		public UInt16 trapezoid_k; // 57
		public UInt32 trapezoid_ctrl; // 58
		public UInt16 trig_delay; // 59
		public UInt16 trig_width; // 59
		public UInt16 upper_discri; // 60
		public UInt16 lower_discri; // 60
		public UInt32 coinc_matrix;
		public UInt32 veto_matrix;
		public double input_min_voltage;  //!< 最小入力電圧(V)
		public double input_max_voltage;  //!< 最大入力電圧(V)
		public int fullscale;
	}

	/// C#で使うための構造体
	public struct UPPStatus2
	{
		public int maxch;
		public int freq;
		public UPPChannelStatus[] ch;
	}

	public struct LockinStatus
	{
		public int iteration;
		public double period;
		public double freq;
		public int refch;
		public int clocks;
		public int maxch;
		public int[] chsel;
		public double[] X;
		public double[] Y;
		public double[] Ampl;
		public double[] Phase;
	}
}
